; pulse_measure_capture_standalone_505.a51: preliminary for IR decoder: displaying pulse width count 5/05
; add ALE assertion for ROM step on big board (11/13)
;  for main computer (assumes 16-bit display, at 8000h, 80001h)
; MODIFIED for STANDALONE: write MSD to P2 rather than ext bus

; BOTH DISPLAY SCHEMES SHOWN: COMMENT OUT THE ONE YOU DON'T WANT.  USE DPTR 16-BIT VERSION FOR TESTING ON YOUR LAB MICRO

; uses CAPTURE function of timer 2 to measure pulse width, displays result

; starts timer on fall of INT0, stops timer on fall of INT1 (=rise of pulse), displays resulting 
;  timer count on displays, lights overflow LED on P1.0 in case timer rolls over

; rev path 3/16
	$NOSYMBOLS					; keeps listing short
	$INCLUDE (C:\SiLabs\MCU\INC\c8051f410.inc) 
	$INCLUDE (C:\SiLabs\MCU\INC\VECTORS320.INC)	; Tom's vectors definition file
	
	
	 STARTFLAG EQU 0	; software flag that indicates edge has fallen 
	 STOPFLAG EQU 1	; ...indicates edge has
	 
	 OVERFLOW EQU TF2	; rollover flag for timer 2
	 OVERFLOW_LED EQU P1.0	; LED that shows overflow
 
	 CKMOD EQU 96h					; address of CKMOD register (a byte)
	 CAP_MODEBIT EQU T2CON.0	; bit that puts timer 2 into capture mode

	 STACKBOT EQU 80h	; scratch RAM, OK place to put stack

	 DIV1 SET 20h		  ; single bit to set hi for prescale to "/1"
	 ENBLTMR2 EQU TR2		; a handy shorthand for this bit
	 

	ORG 0H

	JMP START

	ORG 40h

START: MOV SP, #STACKBOT-1
	 ACALL    INITS         ;GO INITIALIZE

	; ----NOW ENABLE INTERRUPTS----
	SETB IT0		; make INT0 Edge-sensitive	(p. 22)
	SETB EX0		; ...and enable INT0

	SETB EA		; Global int enable        (pp.31-32)


WAITSTART:	JNB STARTFLAG, WAITSTART	; AWAIT start pulse--flag indicates falling edge
	CLR STARTFLAG								; knock down the flag that got us out of the loop
	
WAITSTOP: ACALL SHOWCAP						; display count, from capture registers
	JNB STOPFLAG, WAITSTOP					; AWAIT flag that indicates rising edge
	CLR STOPFLAG								; set up for next pass
	
	SJMP WAITSTART	; 

; SUBROUTINES*******************************************

; SOME INITIALIZATIONS
INITS: 	MOV P2, #0		; clear display, at start
	CLR IE0					; clear int request flags, in case any are latched
	CLR IE1
	CLR STOPFLAG
	CLR OVERFLOW_LED		; turn ON ovflw LED, just to show pgm is alive
	ORL CKMOD, #DIV1		; set div-by-one bit in CKMOD register--COMMMENTED OUT TO SLOW THINGS DOWN
	SETB CAP_MODEBIT		; put timer 2 into Capture mode
	SETB EXEN2				; ...and enable capture
	MOV DPTR, #8000h		; pointer for debugging

	RET

; NEXT THREE LINES SHOW ALTERNATE CODE, FOR USE IN STANDALONE MODE

SHOWCAP:	MOV A, RCAP2H	; get count from capture register
	MOV P2, A				; show hi byte of count
		RET	
		
;SHOWCAP:	MOV A, RCAP2L	; get count from capture register
;	MOVX @DPTR,A
;	MOV A, RCAP2H
;	INC DPTR
;	MOVX @DPTR, A
;	DEC DPL					; restore pointer to usual value
;	RET			
	

DOINT0: 	SETB STARTFLAG
	CLR EX0						; disable further falling-edge interrupts
	SETB OVERFLOW_LED			; turn off overflow LED, for fresh start
	CLR TF2						; clear timer overflow flag
	MOV TH2, #0					; clear counter
	MOV TL2, #0
	SETB ENBLTMR2				; start timer 2
	SETB EX1						; ...and allow terminating interrupt		
	RETI

DOINT1:		SETB STOPFLAG
	CLR EX1					; disable further end-of-pulse interrupts
	CLR STARTFLAG			; (in case rise came and interrupted before clear)
	JNB TF2,VALIDCOUNT	; see if ctr has overflowed
	CLR OVERFLOW_LED		; turn on OVERFLOW LED if flag set
VALIDCOUNT:	CLR ENBLTMR2		; stop timer 2	
	SETB EX0					; enable fall-edge again
	RETI
		
	

; ---ISR'S -------
	ORG INT0VECTOR
	LJMP DOINT0		; out to where we have more space
	
	ORG INT1VECTOR
	LJMP DOINT1
	
	END
